@@ -1,7 +1,7 @@
-/* $Id: time-out-lock-screen.c 2892 2007-07-08 14:18:28Z jannis $ */
 /* vim:set et ai sw=2 sts=2: */
 /*-
  * Copyright (c) 2007 Jannis Pohlmann <jannis@xfce.org>
+ * Copyright (c) 2010 Florian Rivoal <frivoal@xfce.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -36,6 +36,8 @@ static void     time_out_lock_screen_init       (TimeOutLockScreen      *lock_sc
 static void     time_out_lock_screen_finalize   (GObject                *object);
 static void     time_out_lock_screen_postpone   (GtkButton              *button,
                                                  TimeOutLockScreen      *lock_screen);
+static void     time_out_lock_screen_resume     (GtkButton              *button,
+                                                 TimeOutLockScreen      *lock_screen);
 
 
 
@@ -45,21 +47,29 @@ struct _TimeOutLockScreenClass
 
   /* Signals */
   void         (*postpone)  (TimeOutLockScreen *lock_screen);
+  void         (*resume)    (TimeOutLockScreen *lock_screen);
 
   /* Signal identifiers */
   guint        postpone_signal_id;
+  guint        resume_signal_id;
 };
 
 struct _TimeOutLockScreen
 {
   GObject         __parent__;
 
+  /* Total seconds */
+  gint            max_seconds;
+
   /* Remaining seconds */
   gint            remaining_seconds;
 
   /* Whether to allow postpone */
   guint           allow_postpone : 1;
 
+  /* Whether to show the resume button */
+  guint           show_resume : 1;
+
   /* Whether to display seconds */
   guint           display_seconds : 1;
 
@@ -70,6 +80,8 @@ struct _TimeOutLockScreen
   GtkWidget      *window;
   GtkWidget      *time_label;
   GtkWidget      *postpone_button;
+  GtkWidget      *resume_button;
+  GtkWidget      *progress;
 
   /* Fade out */
   TimeOutFadeout *fadeout;
@@ -131,6 +143,17 @@ time_out_lock_screen_class_init (TimeOutLockScreenClass *klass)
                                             g_cclosure_marshal_VOID__VOID,
                                             G_TYPE_NONE,
                                             0);
+
+  /* Register 'resume' signal */
+  klass->resume_signal_id = g_signal_new ("resume",
+                                          G_TYPE_FROM_CLASS (gobject_class),
+                                          G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
+                                          G_STRUCT_OFFSET (TimeOutLockScreenClass, resume),
+                                          NULL,
+                                          NULL,
+                                          g_cclosure_marshal_VOID__VOID,
+                                          G_TYPE_NONE,
+                                          0);
 }
 
 
@@ -146,6 +169,7 @@ time_out_lock_screen_init (TimeOutLockScreen *lock_screen)
 
   lock_screen->display_seconds = TRUE;
   lock_screen->allow_postpone = TRUE;
+  lock_screen->show_resume = FALSE;
   lock_screen->display_hours = FALSE;
   lock_screen->fadeout = NULL;
 
@@ -185,11 +209,22 @@ time_out_lock_screen_init (TimeOutLockScreen *lock_screen)
   gtk_box_pack_start (GTK_BOX (vbox), lock_screen->time_label, FALSE, FALSE, 12);
   gtk_widget_show (lock_screen->time_label);
 
+  /* Create a progress bar to visually display the remaining tme */
+  lock_screen->progress = gtk_progress_bar_new ();
+  gtk_progress_bar_set_orientation (GTK_PROGRESS_BAR (lock_screen->progress), GTK_PROGRESS_LEFT_TO_RIGHT);
+  gtk_box_pack_start (GTK_BOX (vbox), lock_screen->progress, FALSE, FALSE, 0);
+  gtk_widget_show (lock_screen->progress);
+
   /* Create postpone button */
   lock_screen->postpone_button = gtk_button_new_with_mnemonic (_("_Postpone"));
   gtk_box_pack_start (GTK_BOX (vbox), lock_screen->postpone_button, FALSE, FALSE, 0);
   g_signal_connect (G_OBJECT (lock_screen->postpone_button), "clicked", G_CALLBACK (time_out_lock_screen_postpone), lock_screen);
   gtk_widget_show (lock_screen->postpone_button);
+
+  /* Create resume button */
+  lock_screen->resume_button = gtk_button_new_with_mnemonic (_("_Resume"));
+  gtk_box_pack_start (GTK_BOX (vbox), lock_screen->resume_button, FALSE, FALSE, 0);
+  g_signal_connect (G_OBJECT (lock_screen->resume_button), "clicked", G_CALLBACK (time_out_lock_screen_resume), lock_screen);
 }
 
 
@@ -218,7 +253,7 @@ time_out_lock_screen_new (void)
 
 
 void
-time_out_lock_screen_show (TimeOutLockScreen *lock_screen)
+time_out_lock_screen_show (TimeOutLockScreen *lock_screen, gint max_sec)
 {
   GdkScreen *screen;
   gint       monitor;
@@ -242,6 +277,10 @@ time_out_lock_screen_show (TimeOutLockScreen *lock_screen)
   /* Center window on target monitor */
   xfce_gtk_window_center_on_monitor (GTK_WINDOW (lock_screen->window), screen, monitor);
 
+  lock_screen->max_seconds = max_sec;
+
+  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (lock_screen->progress), 1.0);
+
   /* Display information window */
   gtk_widget_show_now (lock_screen->window);
   gtk_widget_grab_focus (lock_screen->window);
@@ -288,6 +327,9 @@ time_out_lock_screen_set_remaining (TimeOutLockScreen *lock_screen,
   /* Update widgets */
   gtk_label_set_markup (GTK_LABEL (lock_screen->time_label), time_string->str);
 
+  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (lock_screen->progress),
+                                 ((gdouble)seconds) / ((gdouble)lock_screen->max_seconds));
+
   /* Free time string */
   g_string_free (time_string, TRUE);
 }
@@ -312,6 +354,24 @@ time_out_lock_screen_set_allow_postpone (TimeOutLockScreen *lock_screen,
 
 
 
+void
+time_out_lock_screen_show_resume (TimeOutLockScreen *lock_screen,
+                                  gboolean           show)
+{
+  g_return_if_fail (IS_TIME_OUT_LOCK_SCREEN (lock_screen));
+
+  /* Set auto resume attribute */
+  lock_screen->show_resume = show;
+
+  /* Enable/disable resume button */
+  if (show)
+    gtk_widget_show (lock_screen->resume_button);
+  else
+    gtk_widget_hide (lock_screen->resume_button);
+}
+
+
+
 void
 time_out_lock_screen_set_display_seconds (TimeOutLockScreen *lock_screen,
                                           gboolean           display_seconds)
@@ -346,3 +406,16 @@ time_out_lock_screen_postpone (GtkButton         *button,
   /* Emit postpone signal */
   g_signal_emit_by_name (lock_screen, "postpone", NULL);
 }
+
+
+
+static void
+time_out_lock_screen_resume (GtkButton         *button,
+                             TimeOutLockScreen *lock_screen)
+{
+  g_return_if_fail (GTK_IS_BUTTON (button));
+  g_return_if_fail (IS_TIME_OUT_LOCK_SCREEN (lock_screen));
+
+  /* Emit resume signal */
+  g_signal_emit_by_name (lock_screen, "resume", NULL);
+}
