@@ -1,7 +1,7 @@
-/* $Id: time-out.c 2892 2007-07-08 14:18:28Z jannis $ */
 /* vim:set et ai sw=2 sts=2: */
 /*-
  * Copyright (c) 2007 Jannis Pohlmann <jannis@xfce.org>
+ * Copyright (c) 2010 Florian Rivoal <frivoal@xfce.org>
  * 
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -38,6 +38,7 @@
 /* Default settings */
 #define DEFAULT_ENABLED                    TRUE
 #define DEFAULT_ALLOW_POSTPONE             TRUE
+#define DEFAULT_AUTO_RESUME                FALSE
 #define DEFAULT_DISPLAY_SECONDS            TRUE
 #define DEFAULT_DISPLAY_HOURS              FALSE
 #define DEFAULT_DISPLAY_TIME               TRUE
@@ -67,6 +68,7 @@ struct _TimeOutPlugin
   guint              display_hours : 1;              
   guint              allow_postpone : 1;
   guint              display_time : 1;
+  guint              auto_resume : 1;
 
   /* Lock screen to be shown during breaks */
   TimeOutLockScreen *lock_screen;
@@ -92,17 +94,24 @@ static gboolean       time_out_size_changed                       (XfcePanelPlug
 static void           time_out_orientation_changed                (XfcePanelPlugin   *plugin,
                                                                    GtkOrientation     orientation,
                                                                    TimeOutPlugin     *time_out);
+static void           time_out_about                              (XfcePanelPlugin   *plugin);
 static void           time_out_configure                          (XfcePanelPlugin   *plugin,
                                                                    TimeOutPlugin     *time_out);
 static void           time_out_end_configure                      (GtkDialog         *dialog,
                                                                    gint               response_id,
                                                                    TimeOutPlugin     *time_out);
+static void           time_out_lock_countdown_minutes_changed     (GtkSpinButton     *spin_button,
+                                                                   TimeOutPlugin     *time_out);
 static void           time_out_lock_countdown_seconds_changed     (GtkSpinButton     *spin_button,
                                                                    TimeOutPlugin     *time_out);
+static void           time_out_postpone_countdown_minutes_changed (GtkSpinButton     *spin_button,
+                                                                   TimeOutPlugin     *time_out);
 static void           time_out_postpone_countdown_seconds_changed (GtkSpinButton     *spin_button,
                                                                    TimeOutPlugin     *time_out);
 static void           time_out_allow_postpone_toggled             (GtkToggleButton   *toggle_button,
                                                                    TimeOutPlugin     *time_out);
+static void           time_out_auto_resume_toggled                (GtkToggleButton   *toggle_button,
+                                                                   TimeOutPlugin     *time_out);
 static void           time_out_display_time_toggled               (GtkToggleButton   *toggle_button,
                                                                    TimeOutPlugin     *time_out);
 static void           time_out_display_seconds_toggled            (GtkToggleButton   *toggle_button,
@@ -115,6 +124,8 @@ static void           time_out_take_break                         (GtkMenuItem
                                                                    TimeOutPlugin     *time_out);
 static void           time_out_enabled_toggled                    (GtkCheckMenuItem  *menu_item,
                                                                    TimeOutPlugin     *time_out);
+static void           time_out_reset_timer                        (GtkMenuItem       *menu_item,
+                                                                   TimeOutPlugin     *time_out);
 static void           time_out_start_break_countdown              (TimeOutPlugin     *time_out,
                                                                    gint               seconds);
 static void           time_out_stop_break_countdown               (TimeOutPlugin     *time_out);
@@ -122,6 +133,8 @@ static void           time_out_start_lock_countdown               (TimeOutPlugin
 static void           time_out_stop_lock_countdown                (TimeOutPlugin     *time_out);
 static void           time_out_postpone                           (TimeOutLockScreen *lock_screen,
                                                                    TimeOutPlugin     *time_out);
+static void           time_out_resume                             (TimeOutLockScreen *lock_screen,
+                                                                   TimeOutPlugin     *time_out);
 static void           time_out_break_countdown_update             (TimeOutCountdown  *countdown,
                                                                    gint               seconds_remaining,
                                                                    TimeOutPlugin     *plugin);
@@ -159,6 +172,9 @@ time_out_new (XfcePanelPlugin *plugin)
   /* Connect to 'postpone' signal of the lock screen */
   g_signal_connect (G_OBJECT (time_out->lock_screen), "postpone", G_CALLBACK (time_out_postpone), time_out);
 
+  /* Connect to 'resume' signal of the lock screen */
+  g_signal_connect (G_OBJECT (time_out->lock_screen), "resume", G_CALLBACK (time_out_resume), time_out);
+
   /* Create countdowns */
   time_out->break_countdown = time_out_countdown_new ();
   time_out->lock_countdown = time_out_countdown_new ();
@@ -238,6 +254,10 @@ time_out_construct (XfcePanelPlugin *plugin)
   /* Load the settings */
   time_out_load_settings (time_out);
 
+  /* Hide the time label if settings says so */
+  if (!time_out->display_time)
+    gtk_widget_hide (time_out->time_label);
+
   /* Add the event box to the panel */
   gtk_container_add (GTK_CONTAINER (plugin), time_out->ebox);
 
@@ -249,6 +269,14 @@ time_out_construct (XfcePanelPlugin *plugin)
   /* Connect to the menu item to react on break requests */
   g_signal_connect (G_OBJECT (menu_item), "activate", G_CALLBACK (time_out_take_break), time_out);
 
+  /* Create menu item for resetting the timer */
+  menu_item = gtk_menu_item_new_with_label (_("Reset timer"));
+  xfce_panel_plugin_menu_insert_item (plugin, GTK_MENU_ITEM (menu_item));
+  gtk_widget_show (GTK_WIDGET (menu_item));
+
+  /* Connect to the menu item to react on reset requests */
+  g_signal_connect (G_OBJECT (menu_item), "activate", G_CALLBACK (time_out_reset_timer), time_out);
+
   /* Create menu item for enabling/disabling the countdown */
   menu_item = gtk_check_menu_item_new_with_label (_("Enabled"));
   xfce_panel_plugin_menu_insert_item (plugin, GTK_MENU_ITEM (menu_item));
@@ -265,6 +293,7 @@ time_out_construct (XfcePanelPlugin *plugin)
   g_signal_connect (G_OBJECT (plugin), "size-changed", G_CALLBACK (time_out_size_changed), time_out);
   g_signal_connect (G_OBJECT (plugin), "configure-plugin", G_CALLBACK (time_out_configure), time_out);
   g_signal_connect (G_OBJECT (plugin), "orientation-changed", G_CALLBACK (time_out_orientation_changed), time_out);
+  g_signal_connect (G_OBJECT (plugin), "about", G_CALLBACK (time_out_about), NULL);
 
   /* Display the configure menu item */
   xfce_panel_plugin_menu_show_configure (plugin);
@@ -320,7 +349,23 @@ time_out_enabled_toggled (GtkCheckMenuItem *menu_item,
   time_out_save_settings (time_out);
 }
 
+static void 
+time_out_reset_timer(GtkMenuItem *menu_item,
+                    TimeOutPlugin    *time_out)
+{
+  g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
+  g_return_if_fail (time_out != NULL);
 
+  time_out_countdown_stop (time_out->break_countdown);
+  time_out_countdown_start (time_out->break_countdown, time_out->break_countdown_seconds);
+  /* if we're not enabled, we need to update the timer */
+  if(!time_out->enabled)
+  {
+      time_out_break_countdown_update(time_out->break_countdown, time_out->break_countdown_seconds, time_out);
+      /* keep it paused, however */
+      time_out_countdown_pause (time_out->break_countdown);
+  }
+}
 
 static gboolean
 time_out_size_changed (XfcePanelPlugin *plugin,
@@ -362,7 +407,32 @@ time_out_orientation_changed (XfcePanelPlugin *plugin,
   xfce_hvbox_set_orientation (XFCE_HVBOX (time_out->hvbox), orientation);
 }
 
+static void
+time_out_about (XfcePanelPlugin *plugin)
+{
+  static const gchar *authors[] =
+  {
+    "Jannis Pohlmann <jannis@xfce.org>",
+    "Florian Rivoal <frivoal@xfce.org>",
+    NULL
+  };
 
+  gtk_show_about_dialog (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (plugin))),
+                         "authors", authors,
+                         "comments", _("Xfce Panel plugin for taking a break from computer work every now and then."),
+                         "destroy-with-parent", TRUE,
+                         "logo-icon-name", "xfce4-time-out-plugin",
+#if GTK_CHECK_VERSION (2,12,0)
+                         "program-name", PACKAGE_NAME,
+#else
+                         "name", PACKAGE_NAME,
+#endif
+                         "version", PACKAGE_VERSION,
+                         "translator-credits", _("translator-credits"),
+                         "license", XFCE_LICENSE_GPL,
+                         "website", "http://goodies.xfce.org/projects/panel-plugins/xfce4-time-out-plugin",
+                         NULL);
+}
 
 static void
 time_out_configure (XfcePanelPlugin *plugin,
@@ -413,51 +483,81 @@ time_out_configure (XfcePanelPlugin *plugin,
   gtk_widget_show (framebox);
 
   /* Create time settings table */
-  table = gtk_table_new (2, 2, FALSE);
+  table = gtk_table_new (3, 3, FALSE);
   gtk_table_set_row_spacings (GTK_TABLE (table), 6);
   gtk_table_set_col_spacings (GTK_TABLE (table), 12);
   gtk_container_add (GTK_CONTAINER (timebin), table);
   gtk_widget_show (table);
 
-  /* Create break countdown time label */
-  label = gtk_label_new (_("Time between breaks (minutes):"));
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
-  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 0, 1);
+  /* Create the labels for the minutes and seconds spins */
+  label = gtk_label_new (_("Minutes"));
+  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+  gtk_table_attach (GTK_TABLE (table), label, 1, 2, 0, 1, GTK_SHRINK, GTK_FILL, 1.0, 0.5);
   gtk_widget_show (label);
 
-  /* Create break countdown time spin */
+  label = gtk_label_new (_("Seconds"));
+  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+  gtk_table_attach (GTK_TABLE (table), label, 2, 3, 0, 1, GTK_SHRINK, GTK_FILL, 1.0, 0.5);
+  gtk_widget_show (label);
+
+  /* Create break countdown time label */
+  label = gtk_label_new (_("Time between breaks:"));
+  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
+  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_SHRINK, GTK_FILL, 1.0, 0.5);
+  gtk_widget_show (label);
+
+  /* Create break countdown time minute spin */
   spin = gtk_spin_button_new_with_range (1, 24 * 60, 1);
   gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin), time_out->break_countdown_seconds / 60);
-  gtk_table_attach_defaults (GTK_TABLE (table), spin, 1, 2, 0, 1);
+  gtk_table_attach (GTK_TABLE (table), spin, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 0.0, 0.5);
+  gtk_widget_show (spin);
+
+  /* Store reference on the spin button in the plugin */
+  g_object_set_data (G_OBJECT (time_out->plugin), "break-countdown-minutes-spin", spin);
+
+  /* Create break countdown time minute spin */
+  spin = gtk_spin_button_new_with_range (0, 59, 1);
+  gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin), time_out->break_countdown_seconds % 60);
+  gtk_table_attach (GTK_TABLE (table), spin, 2, 3, 1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 0.0, 0.5);
   gtk_widget_show (spin);
 
   /* Store reference on the spin button in the plugin */
   g_object_set_data (G_OBJECT (time_out->plugin), "break-countdown-seconds-spin", spin);
 
   /* Create lock countdown time label */
-  label = gtk_label_new (_("Break length (minutes):"));
+  label = gtk_label_new (_("Break length:"));
   gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
-  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 1, 2);
+  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3, GTK_SHRINK, GTK_FILL, 1.0, 0.5);
   gtk_widget_show (label);
 
-  /* Create lock countdown time spin */
-  spin = gtk_spin_button_new_with_range (1, 24 * 60, 1);
+  /* Create lock countdown time spins */
+  spin = gtk_spin_button_new_with_range (0, 24 * 60, 1);
   gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin), time_out->lock_countdown_seconds / 60);
+  g_signal_connect (spin, "value-changed", G_CALLBACK (time_out_lock_countdown_minutes_changed), time_out);
+  gtk_table_attach (GTK_TABLE (table), spin, 1, 2, 2, 3, GTK_EXPAND | GTK_FILL, GTK_FILL, 0.0, 0.5);
+  gtk_widget_show (spin);
+  spin = gtk_spin_button_new_with_range (0, 59, 1);
+  gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin), time_out->lock_countdown_seconds % 60);
   g_signal_connect (spin, "value-changed", G_CALLBACK (time_out_lock_countdown_seconds_changed), time_out);
-  gtk_table_attach_defaults (GTK_TABLE (table), spin, 1, 2, 1, 2);
+  gtk_table_attach (GTK_TABLE (table), spin, 2, 3, 2, 3, GTK_EXPAND | GTK_FILL, GTK_FILL, 0.0, 0.5);
   gtk_widget_show (spin);
 
   /* Create postpone countdown time label */
-  label = gtk_label_new (_("Postpone length (minutes):"));
+  label = gtk_label_new (_("Postpone length:"));
   gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
-  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 2, 3);
+  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4, GTK_SHRINK, GTK_FILL, 1.0, 0.5);
   gtk_widget_show (label);
 
-  /* Create postpone countdown time spin */
-  spin = gtk_spin_button_new_with_range (1, 24 * 60, 1);
+  /* Create postpone countdown time spins */
+  spin = gtk_spin_button_new_with_range (0, 24 * 60, 1);
   gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin), time_out->postpone_countdown_seconds / 60);
+  g_signal_connect (spin, "value-changed", G_CALLBACK (time_out_postpone_countdown_minutes_changed), time_out);
+  gtk_table_attach (GTK_TABLE (table), spin, 1, 2, 3, 4, GTK_EXPAND | GTK_FILL, GTK_FILL, 0.0, 0.5);
+  gtk_widget_show (spin);
+  spin = gtk_spin_button_new_with_range (0, 59, 1);
+  gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin), time_out->postpone_countdown_seconds % 60);
   g_signal_connect (spin, "value-changed", G_CALLBACK (time_out_postpone_countdown_seconds_changed), time_out);
-  gtk_table_attach_defaults (GTK_TABLE (table), spin, 1, 2, 2, 3);
+  gtk_table_attach (GTK_TABLE (table), spin, 2, 3, 3, 4, GTK_EXPAND | GTK_FILL, GTK_FILL, 0.0, 0.5);
   gtk_widget_show (spin);
 
   /* Create behaviour section */
@@ -466,11 +566,23 @@ time_out_configure (XfcePanelPlugin *plugin,
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), framebox, TRUE, TRUE, 0);
   gtk_widget_show (framebox);
 
+  /* Create behaviour box */
+  vbox = gtk_vbox_new (FALSE, 6);
+  gtk_container_add (GTK_CONTAINER (behaviourbin), vbox);
+  gtk_widget_show (vbox);
+
   /* Create postpone check button */
   checkbutton = gtk_check_button_new_with_label (_("Allow postpone"));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton), time_out->allow_postpone);
   g_signal_connect (checkbutton, "toggled", G_CALLBACK (time_out_allow_postpone_toggled), time_out);
-  gtk_container_add (GTK_CONTAINER (behaviourbin), checkbutton);
+  gtk_container_add (GTK_CONTAINER (vbox), checkbutton);
+  gtk_widget_show (checkbutton);
+
+  /* Create resume check button */
+  checkbutton = gtk_check_button_new_with_label (_("Resume automatically"));
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton), time_out->auto_resume);
+  g_signal_connect (checkbutton, "toggled", G_CALLBACK (time_out_auto_resume_toggled), time_out);
+  gtk_container_add (GTK_CONTAINER (vbox), checkbutton);
   gtk_widget_show (checkbutton);
 
   /* Create appearance section */
@@ -527,8 +639,12 @@ time_out_end_configure (GtkDialog     *dialog,
   xfce_panel_plugin_unblock_menu (time_out->plugin);
 
   /* Get spin button value for the break countdown settings */
-  spin = g_object_get_data (G_OBJECT (time_out->plugin), "break-countdown-seconds-spin");
+  spin = g_object_get_data (G_OBJECT (time_out->plugin), "break-countdown-minutes-spin");
   value = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spin)) * 60;
+  g_object_set_data (G_OBJECT (time_out->plugin), "break-countdown-minutes-spin", NULL);
+
+  spin = g_object_get_data (G_OBJECT (time_out->plugin), "break-countdown-seconds-spin");
+  value += gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spin));
   g_object_set_data (G_OBJECT (time_out->plugin), "break-countdown-seconds-spin", NULL);
 
   /* Check if the break countdown seconds have changed */
@@ -537,6 +653,13 @@ time_out_end_configure (GtkDialog     *dialog,
   /* Apply new break countdown seconds */
   time_out->break_countdown_seconds = value;
 
+  /* prevent 0 seconds values */
+  if (!time_out->lock_countdown_seconds)
+    time_out->lock_countdown_seconds = 1;
+
+  if (!time_out->postpone_countdown_seconds)
+    time_out->postpone_countdown_seconds = 1;
+
   /* Save plugin configuration */
   time_out_save_settings (time_out);
 
@@ -555,6 +678,17 @@ time_out_end_configure (GtkDialog     *dialog,
 
 
 
+static void
+time_out_lock_countdown_minutes_changed (GtkSpinButton *spin_button,
+                                         TimeOutPlugin *time_out)
+{
+  g_return_if_fail (GTK_IS_SPIN_BUTTON (spin_button));
+  g_return_if_fail (time_out != NULL);
+
+  /* Set plugin attribute */
+  time_out->lock_countdown_seconds = gtk_spin_button_get_value_as_int (spin_button) * 60 + time_out->lock_countdown_seconds % 60;
+}
+
 static void
 time_out_lock_countdown_seconds_changed (GtkSpinButton *spin_button,
                                          TimeOutPlugin *time_out)
@@ -563,10 +697,20 @@ time_out_lock_countdown_seconds_changed (GtkSpinButton *spin_button,
   g_return_if_fail (time_out != NULL);
 
   /* Set plugin attribute */
-  time_out->lock_countdown_seconds = gtk_spin_button_get_value_as_int (spin_button) * 60;
+  time_out->lock_countdown_seconds = time_out->lock_countdown_seconds / 60 * 60 + gtk_spin_button_get_value_as_int (spin_button);
 }
 
 
+static void
+time_out_postpone_countdown_minutes_changed (GtkSpinButton *spin_button,
+                                             TimeOutPlugin *time_out)
+{
+  g_return_if_fail (GTK_IS_SPIN_BUTTON (spin_button));
+  g_return_if_fail (time_out != NULL);
+
+  /* Set plugin attribute */
+  time_out->postpone_countdown_seconds = gtk_spin_button_get_value_as_int (spin_button) * 60 + time_out->postpone_countdown_seconds % 60;
+}
 
 static void
 time_out_postpone_countdown_seconds_changed (GtkSpinButton *spin_button,
@@ -576,7 +720,7 @@ time_out_postpone_countdown_seconds_changed (GtkSpinButton *spin_button,
   g_return_if_fail (time_out != NULL);
 
   /* Set plugin attribute */
-  time_out->postpone_countdown_seconds = gtk_spin_button_get_value_as_int (spin_button) * 60;
+  time_out->postpone_countdown_seconds = time_out->postpone_countdown_seconds / 60 * 60 +gtk_spin_button_get_value_as_int (spin_button);
 }
 
 
@@ -594,6 +738,19 @@ time_out_allow_postpone_toggled (GtkToggleButton *toggle_button,
 
 
 
+static void
+time_out_auto_resume_toggled (GtkToggleButton *toggle_button,
+                              TimeOutPlugin   *time_out)
+{
+  g_return_if_fail (GTK_IS_TOGGLE_BUTTON (toggle_button));
+  g_return_if_fail (time_out != NULL);
+
+  /* Set resume attribute */
+  time_out->auto_resume = gtk_toggle_button_get_active (toggle_button);
+}
+
+
+
 static void 
 time_out_display_time_toggled (GtkToggleButton *toggle_button,
                                TimeOutPlugin   *time_out)
@@ -654,6 +811,7 @@ time_out_load_settings (TimeOutPlugin *time_out)
   gboolean display_hours = DEFAULT_DISPLAY_HOURS;
   gboolean display_time = DEFAULT_DISPLAY_TIME;
   gboolean allow_postpone = DEFAULT_ALLOW_POSTPONE;
+  gboolean auto_resume = DEFAULT_AUTO_RESUME;;
 
   g_return_if_fail (time_out != NULL);
 
@@ -678,6 +836,7 @@ time_out_load_settings (TimeOutPlugin *time_out)
           display_hours = xfce_rc_read_bool_entry (rc, "display-hours", display_hours);
           display_time = xfce_rc_read_bool_entry (rc, "display-time", display_time);
           allow_postpone = xfce_rc_read_bool_entry (rc, "allow-postpone", allow_postpone);
+          auto_resume = xfce_rc_read_bool_entry (rc, "auto-resume", auto_resume);
 
           /* Close file handle */
           xfce_rc_close (rc);
@@ -696,6 +855,7 @@ time_out_load_settings (TimeOutPlugin *time_out)
   time_out->display_hours = display_hours;
   time_out->display_time = display_time;
   time_out->allow_postpone = allow_postpone;
+  time_out->auto_resume = auto_resume;
 }
 
 
@@ -728,6 +888,8 @@ time_out_save_settings (TimeOutPlugin *time_out)
           xfce_rc_write_bool_entry (rc, "display-seconds", time_out->display_seconds);
           xfce_rc_write_bool_entry (rc, "display-hours", time_out->display_hours);
           xfce_rc_write_bool_entry (rc, "display-time", time_out->display_time);
+          xfce_rc_write_bool_entry (rc, "allow-postpone", time_out->allow_postpone);
+          xfce_rc_write_bool_entry (rc, "auto-resume", time_out->auto_resume);
 
           /* Close file handle */
           xfce_rc_close (rc);
@@ -772,8 +934,11 @@ time_out_start_lock_countdown (TimeOutPlugin *time_out)
   /* Set whether to allow postpone or not */
   time_out_lock_screen_set_allow_postpone (time_out->lock_screen, time_out->allow_postpone);
 
+  /* Hide the resume button initially */
+  time_out_lock_screen_show_resume (time_out->lock_screen, FALSE);
+
   /* Display the lock screen */
-  time_out_lock_screen_show (time_out->lock_screen);
+  time_out_lock_screen_show (time_out->lock_screen, time_out->lock_countdown_seconds);
 }
 
 
@@ -819,6 +984,22 @@ time_out_postpone (TimeOutLockScreen *lock_screen,
 
 
 
+static void
+time_out_resume   (TimeOutLockScreen *lock_screen,
+                   TimeOutPlugin     *time_out)
+{
+  g_return_if_fail (IS_TIME_OUT_LOCK_SCREEN (lock_screen));
+  g_return_if_fail (time_out != NULL);
+
+  /* Stop lock countdown */
+  time_out_stop_lock_countdown (time_out);
+
+  /* Start break countdown */
+  time_out_start_break_countdown (time_out, time_out->break_countdown_seconds);
+}
+
+
+
 static void 
 time_out_break_countdown_update (TimeOutCountdown *countdown,
                                  gint              seconds_remaining,
@@ -882,9 +1063,18 @@ time_out_lock_countdown_finish (TimeOutCountdown *countdown,
   g_return_if_fail (IS_TIME_OUT_COUNTDOWN (countdown));
   g_return_if_fail (time_out != NULL);
 
-  /* Stop lock countdown */
-  time_out_stop_lock_countdown (time_out);
+  if (time_out->auto_resume)
+  {
+    /* Stop lock countdown */
+    time_out_stop_lock_countdown (time_out);
 
-  /* Start break countdown */
-  time_out_start_break_countdown (time_out, time_out->break_countdown_seconds);
+    /* Start break countdown */
+    time_out_start_break_countdown (time_out, time_out->break_countdown_seconds);
+  }
+  else
+  {
+    time_out_lock_screen_set_remaining (time_out->lock_screen, 0);
+    time_out_lock_screen_set_allow_postpone (time_out->lock_screen, FALSE);
+    time_out_lock_screen_show_resume (time_out->lock_screen, TRUE);
+  }
 }
